import numpy as np
from sklearn.feature_extraction.text import CountVectorizer
from tqdm import tqdm
from util import parallelize


class Index:
    def __init__(self, **kwargs):
        """
        :param kwargs: keyworded arguments from _sklearn.feature_extraction.text.CountVectorizer_
        """
        self.vect = CountVectorizer(**kwargs)
        self.unk = -1  # a valid index is assigned after fit

    def fit(self, X):
        """
        :param X: a list of strings
        :return: self
        """
        self.vect.fit(X)
        self.analyzer = self.vect.build_analyzer()
        self.vocabulary = self.vect.vocabulary_
        self.unk = self.add_word('UNKTOKEN')
        return self

    def transform(self, X, n_jobs=-1):
        assert self.unk > 0, 'transform called before fit'
        indexed = parallelize(func=self.index, args=[doc for doc in X], n_jobs=n_jobs)
        return np.asarray(indexed)

    def index(self, documents):
        return [[self.vocabulary.get(word, self.unk) for word in self.analyzer(doc)] for doc in tqdm(documents, 'indexing')]

    def fit_transform(self, X):
        return self.fit(X).transform(X)

    def vocabulary_size(self):
        return len(self.vocabulary) + 1  # the reserved unk token

    def add_word(self, word):
        if word in self.vocabulary:
            raise ValueError(f'word {word} already in dictionary')
        self.vocabulary[word] = len(self.vocabulary)
        return self.vocabulary[word]










