from pathlib import Path
import os
from joblib import Parallel, delayed
import multiprocessing
import itertools
import pickle



def create_path_if_not_exists(file):
    parent_path = Path(file).parent
    os.makedirs(parent_path, exist_ok=True)


# given the number of tasks and the number of jobs, generates the slices for the parallel threads
def get_parallel_slices(n_tasks, n_jobs=-1):
    if n_jobs==-1:
        n_jobs = multiprocessing.cpu_count()
    batch = int(n_tasks / n_jobs)
    remainder = n_tasks % n_jobs
    return [slice(job*batch, (job+1)*batch + (remainder if job == n_jobs - 1 else 0)) for job in range(n_jobs)]


def parallelize(func, args, n_jobs):
    slices = get_parallel_slices(len(args), n_jobs)

    results = Parallel(n_jobs=n_jobs)(
        delayed(func)(args[slice_i]) for slice_i in slices
    )
    return list(itertools.chain.from_iterable(results))



def pickled_resource(pickle_path: str, generation_func: callable, *args, **kwargs):
    if pickle_path is None:
        return generation_func(*args, **kwargs)
    else:
        if os.path.exists(pickle_path):
            return pickle.load(open(pickle_path, 'rb'))
        else:
            instance = generation_func(*args, **kwargs)
            os.makedirs(str(Path(pickle_path).parent), exist_ok=True)
            pickle.dump(instance, open(pickle_path, 'wb'), pickle.HIGHEST_PROTOCOL)
            return instance


